searchData={"items":[{"type":"module","title":"Bandit","doc":"Bandit is an HTTP server for Plug and WebSock apps.\n\nAs an HTTP server, Bandit's primary goal is to act as 'glue' between client connections managed\nby [Thousand Island](https://github.com/mtrudel/thousand_island) and application code defined\nvia the [Plug](https://github.com/elixir-plug/plug) and/or\n[WebSock](https://github.com/phoenixframework/websock) APIs. As such there really isn't a whole lot of\nuser-visible surface area to Bandit, and as a consequence the API documentation presented here\nis somewhat sparse. This is by design! Bandit is intended to 'just work' in almost all cases;\nthe only thought users typically have to put into Bandit comes in the choice of which options (if\nany) they would like to change when starting a Bandit server. The sparseness of the Bandit API\nshould not be taken as an indicator of the comprehensiveness or robustness of the project.","ref":"Bandit.html"},{"type":"module","title":"Using Bandit With Phoenix - Bandit","doc":"Bandit fully supports Phoenix. Phoenix applications which use WebSockets for\nfeatures such as Channels or LiveView require Phoenix 1.7 or later.\n\nUsing Bandit to host your Phoenix application couldn't be simpler:\n\n1. Add Bandit as a dependency in your Phoenix application's `mix.exs`:\n\n    ```elixir\n    {:bandit, \"~> 1.0\"}\n    ```\n2. Add the following `adapter:` line to your endpoint configuration in `config/config.exs`, as in the following example:\n\n     ```elixir\n     # config/config.exs\n\n     config :your_app, YourAppWeb.Endpoint,\n       adapter: Bandit.PhoenixAdapter, # <---- ADD THIS LINE\n       url: [host: \"localhost\"],\n       render_errors: ...\n     ```\n3. That's it! **You should now see messages at startup indicating that Phoenix is\n   using Bandit to serve your endpoint**, and everything should 'just work'. Note\n   that if you have set any exotic configuration options within your endpoint,\n   you may need to update that configuration to work with Bandit; see the\n   [Bandit.PhoenixAdapter](https://hexdocs.pm/bandit/Bandit.PhoenixAdapter.html)\n   documentation for more information.","ref":"Bandit.html#module-using-bandit-with-phoenix"},{"type":"module","title":"Using Bandit With Plug Applications - Bandit","doc":"Using Bandit to host your own Plug is very straightforward. Assuming you have\na Plug module implemented already, you can host it within Bandit by adding\nsomething similar to the following to your application's `Application.start/2`\nfunction:\n\n```elixir\n# lib/my_app/application.ex\n\ndefmodule MyApp.Application do\n  use Application\n\n  def start(_type, _args) do\n    children = [\n      {Bandit, plug: MyApp.MyPlug}\n    ]\n\n    opts = [strategy: :one_for_one, name: MyApp.Supervisor]\n    Supervisor.start_link(children, opts)\n  end\nend\n```\n\nFor less formal usage, you can also start Bandit using the same configuration\noptions via the `Bandit.start_link/1` function:\n\n```elixir\n# Start an http server on the default port 4000, serving MyApp.MyPlug\nBandit.start_link(plug: MyPlug)\n```","ref":"Bandit.html#module-using-bandit-with-plug-applications"},{"type":"module","title":"Configuration - Bandit","doc":"A number of options are defined when starting a server. The complete list is\ndefined by the [`t:Bandit.options/0`](https://hexdocs.pm/bandit/Bandit.html#summary) type.","ref":"Bandit.html#module-configuration"},{"type":"module","title":"Setting up an HTTPS Server - Bandit","doc":"By far the most common stumbling block encountered when setting up an HTTPS\nserver involves configuring key and certificate data. Bandit is comparatively\neasy to set up in this regard, with a working example looking similar to the\nfollowing:\n\n```elixir\n# lib/my_app/application.ex\n\ndefmodule MyApp.Application do\n  use Application\n\n  def start(_type, _args) do\n    children = [\n      {Bandit,\n       plug: MyApp.MyPlug,\n       scheme: :https,\n       certfile: \"/absolute/path/to/cert.pem\",\n       keyfile: \"/absolute/path/to/key.pem\"}\n    ]\n\n    opts = [strategy: :one_for_one, name: MyApp.Supervisor]\n    Supervisor.start_link(children, opts)\n  end\nend\n```","ref":"Bandit.html#module-setting-up-an-https-server"},{"type":"module","title":"WebSocket Support - Bandit","doc":"If you're using Bandit to run a Phoenix application as suggested above, there is\nnothing more for you to do; WebSocket support will 'just work'.\n\nIf you wish to interact with WebSockets at a more fundamental level, the\n[WebSock](https://hexdocs.pm/websock/WebSock.html) and\n[WebSockAdapter](https://hexdocs.pm/websock_adapter/WebSockAdapter.html) libraries\nprovides a generic abstraction for WebSockets (very similar to how Plug is\na generic abstraction on top of HTTP). Bandit fully supports all aspects of\nthese libraries.","ref":"Bandit.html#module-websocket-support"},{"type":"function","title":"Bandit.start_link/1","doc":"Starts a Bandit server using the provided arguments. See `t:options/0` for specific options to\npass to this function.","ref":"Bandit.html#start_link/1"},{"type":"type","title":"Bandit.deflate_options/0","doc":"Options to configure the deflate library used for HTTP compression","ref":"Bandit.html#t:deflate_options/0"},{"type":"type","title":"Bandit.http_1_options/0","doc":"Options to configure the HTTP/1 stack in Bandit\n\n* `enabled`: Whether or not to serve HTTP/1 requests. Defaults to true\n* `max_request_line_length`: The maximum permitted length of the request line\n  (expressed as the number of bytes on the wire) in an HTTP/1.1 request. Defaults to 10_000 bytes\n* `max_header_length`: The maximum permitted length of any single header (combined\n  key & value, expressed as the number of bytes on the wire) in an HTTP/1.1 request. Defaults to 10_000 bytes\n* `max_header_count`: The maximum permitted number of headers in an HTTP/1.1 request.\n  Defaults to 50 headers\n* `max_requests`: The maximum number of requests to serve in a single\n  HTTP/1.1 connection before closing the connection. Defaults to 0 (no limit)\n* `gc_every_n_keepalive_requests`: How often to run a full garbage collection pass between subsequent\n  keepalive requests on the same HTTP/1.1 connection. Defaults to 5 (garbage collect between\n  every 5 requests). This option is currently experimental, and may change at any time\n* `log_unknown_messages`: Whether or not to log unknown messages sent to the handler process.\n  Defaults to `false`","ref":"Bandit.html#t:http_1_options/0"},{"type":"type","title":"Bandit.http_2_options/0","doc":"Options to configure the HTTP/2 stack in Bandit\n\n* `enabled`: Whether or not to serve HTTP/2 requests. Defaults to true\n* `max_header_block_size`: The maximum permitted length of a field block of an HTTP/2 request\n  (expressed as the number of compressed bytes). Includes any concatenated block fragments from\n  continuation frames. Defaults to 50_000 bytes\n* `max_requests`: The maximum number of requests to serve in a single\n  HTTP/2 connection before closing the connection. Defaults to 0 (no limit)\n* `default_local_settings`: Options to override the default values for local HTTP/2\n  settings. Values provided here will override the defaults specified in RFC9113§6.5.2","ref":"Bandit.html#t:http_2_options/0"},{"type":"type","title":"Bandit.http_options/0","doc":"Options to configure shared aspects of the HTTP stack in Bandit\n\n* `compress`: Whether or not to attempt compression of responses via content-encoding\n  negotiation as described in\n  [RFC9110§8.4](https://www.rfc-editor.org/rfc/rfc9110.html#section-8.4). Defaults to true\n* `deflate_options`: A keyword list of options to set on the deflate library. A complete list can\n  be found at `t:deflate_options/0`\n* `log_protocol_errors`: Whether or not to log protocol errors such as malformed requests.\n  Defaults to `true`","ref":"Bandit.html#t:http_options/0"},{"type":"type","title":"Bandit.options/0","doc":"Possible top-level options to configure a Bandit server\n\n* `plug`: The Plug to use to handle connections. Can be specified as `MyPlug` or `{MyPlug, plug_opts}`\n* `scheme`: One of `:http` or `:https`. If `:https` is specified, you will also need to specify\n  valid `certfile` and `keyfile` values (or an equivalent value within\n  `thousand_island_options.transport_options`). Defaults to `:http`\n* `port`: The TCP port to listen on. This option is offered as a convenience and actually sets\n  the option of the same name within `thousand_island_options`. If a string value is passed, it\n  will be parsed as an integer. Defaults to 4000 if `scheme` is `:http`, and 4040 if `scheme` is\n  `:https`\n* `ip`:  The interface(s) to listen on. This option is offered as a convenience and actually sets the\n  option of the same name within `thousand_island_options.transport_options`. Can be specified as:\n    * `{1, 2, 3, 4}` for IPv4 addresses\n    * `{1, 2, 3, 4, 5, 6, 7, 8}` for IPv6 addresses\n    * `:loopback` for local loopback (ie: `127.0.0.1`)\n    * `:any` for all interfaces (ie: `0.0.0.0`)\n    * `{:local, \"/path/to/socket\"}` for a Unix domain socket. If this option is used, the `port`\n      option *must* be set to `0`\n* `keyfile`: The path to a file containing the SSL key to use for this server. This option is\n  offered as a convenience and actually sets the option of the same name within\n  `thousand_island_options.transport_options`. If a relative path is used here, you will also\n  need to set the `otp_app` parameter and ensure that the named file is part of your application\n  build\n* `certfile`: The path to a file containing the SSL certificate to use for this server. This option is\n  offered as a convenience and actually sets the option of the same name within\n  `thousand_island_options.transport_options`. If a relative path is used here, you will also\n  need to set the `otp_app` parameter and ensure that the named file is part of your application\n  build\n* `otp_app`: Provided as a convenience when using relative paths for `keyfile` and `certfile`\n* `cipher_suite`: Used to define a pre-selected set of ciphers, as described by\n  `Plug.SSL.configure/1`. Optional, can be either `:strong` or `:compatible`\n* `display_plug`: The plug to use when describing the connection in logs. Useful for situations\n  such as Phoenix code reloading where you have a 'wrapper' plug but wish to refer to the\n  connection by the endpoint name\n* `startup_log`: The log level at which Bandit should log startup info.\n  Defaults to `:info` log level, can be set to false to disable it\n* `thousand_island_options`: A list of options to pass to Thousand Island. Bandit sets some\n  default values in this list based on your top-level configuration; these values will be\n  overridden by values appearing here. A complete list can be found at\n  `t:ThousandIsland.options/0`\n* `http_options`: A list of options to configure the shared aspects of Bandit's HTTP stack. A\n  complete list can be found at `t:http_options/0`\n* `http_1_options`: A list of options to configure Bandit's HTTP/1 stack. A complete list can\n  be found at `t:http_1_options/0`\n* `http_2_options`: A list of options to configure Bandit's HTTP/2 stack. A complete list can\n  be found at `t:http_2_options/0`\n* `websocket_options`: A list of options to configure Bandit's WebSocket stack. A complete list can\n  be found at `t:websocket_options/0`","ref":"Bandit.html#t:options/0"},{"type":"type","title":"Bandit.websocket_options/0","doc":"Options to configure the WebSocket stack in Bandit\n\n* `enabled`: Whether or not to serve WebSocket upgrade requests. Defaults to true\n* `max_frame_size`: The maximum size of a single WebSocket frame (expressed as\n  a number of bytes on the wire). Defaults to 0 (no limit)\n* `validate_text_frames`: Whether or not to validate text frames as being UTF-8. Strictly\n  speaking this is required per RFC6455§5.6, however it can be an expensive operation and one\n  that may be safely skipped in some situations. Defaults to true\n* `compress`: Whether or not to allow per-message deflate compression globally. Note that\n  upgrade requests still need to set the `compress: true` option in `connection_opts` on\n  a per-upgrade basis for compression to be negotiated (see 'WebSocket Support' section below\n  for details). Defaults to `true`","ref":"Bandit.html#t:websocket_options/0"},{"type":"module","title":"Bandit.HTTP2.Settings","doc":"Settings as defined in RFC9113§6.5.2","ref":"Bandit.HTTP2.Settings.html"},{"type":"type","title":"Bandit.HTTP2.Settings.t/0","doc":"A collection of settings as defined in RFC9113§6.5","ref":"Bandit.HTTP2.Settings.html#t:t/0"},{"type":"module","title":"Bandit.Logger","doc":"Logging conveniences for Bandit servers\n\nAllows dynamically adding and altering the log level used to trace connections\nwithin a Bandit server via the use of telemetry hooks. Should you wish\nto do your own logging or tracking of these events, a complete list of the\ntelemetry events emitted by Bandit is described in the module documentation\nfor `Bandit.Telemetry`.\n\nThe logging included in this module is concerned specifically with protocol level events.\nShould you wish to log lower level transport concerns, there are similar functions to these in\nthe `ThousandIsland.Logger` module. Corresponding telemetry events are described in the\nmodule documentation for `ThousandIsland.Telemetry`.","ref":"Bandit.Logger.html"},{"type":"function","title":"Bandit.Logger.attach_logger/1","doc":"Start logging Bandit at the specified log level. Valid values for log\nlevel are `:error` and `:info`. Enabling a given log level implicitly enables all higher log\nlevels as well.","ref":"Bandit.Logger.html#attach_logger/1"},{"type":"function","title":"Bandit.Logger.detach_logger/1","doc":"Stop logging Thousand Island at the specified log level. Disabling a given log\nlevel implicitly disables all lower log levels as well.","ref":"Bandit.Logger.html#detach_logger/1"},{"type":"type","title":"Bandit.Logger.log_level/0","doc":"Supported log levels","ref":"Bandit.Logger.html#t:log_level/0"},{"type":"module","title":"Bandit.PhoenixAdapter","doc":"A Bandit adapter for Phoenix.\n\nThis adapter provides out-of-the-box support for all aspects of Phoenix 1.7 and later. Earlier\nversions of Phoenix will work with this adapter, but without support for WebSockets.\n\nTo use this adapter, your project will need to include Bandit as a dependency:\n\n```elixir\n{:bandit, \"~> 1.0\"}\n```\n\nOnce Bandit is included as a dependency of your Phoenix project, add the following `adapter:`\nline to your endpoint configuration in `config/config.exs`, as in the following example:\n\n```\n# config/config.exs\n\nconfig :your_app, YourAppWeb.Endpoint,\n  adapter: Bandit.PhoenixAdapter, # <---- ADD THIS LINE\n  url: [host: \"localhost\"],\n  render_errors: ...\n```\n\nThat's it! **After restarting Phoenix you should see the startup message indicate that it is being\nserved by Bandit**, and everything should 'just work'. Note that if you have set any exotic\nconfiguration options within your endpoint, you may need to update that configuration to work\nwith Bandit; see below for details.","ref":"Bandit.PhoenixAdapter.html"},{"type":"module","title":"Endpoint configuration - Bandit.PhoenixAdapter","doc":"This adapter supports the standard Phoenix structure for endpoint configuration. Top-level keys for\n`:http` and `:https` are supported, and configuration values within each of those are interpreted\nas raw Bandit configuration as specified by `t:Bandit.options/0`. Bandit's confguration supports\nall values used in a standard out-of-the-box Phoenix application, so if you haven't made any\nsubstantial changes to your endpoint configuration things should 'just work' for you.\n\nIn the event that you *have* made advanced changes to your endpoint configuration, you may need\nto update this config to work with Bandit. Consult Bandit's documentation at\n`t:Bandit.options/0` for details.","ref":"Bandit.PhoenixAdapter.html#module-endpoint-configuration"},{"type":"function","title":"Bandit.PhoenixAdapter.bandit_pid/2","doc":"Returns the Bandit server proccess for the provided scheme within the given Phoenix Endpoint","ref":"Bandit.PhoenixAdapter.html#bandit_pid/2"},{"type":"function","title":"Bandit.PhoenixAdapter.server_info/2","doc":"Returns the bound address and port of the Bandit server process for the provided\nscheme within the given Phoenix Endpoint","ref":"Bandit.PhoenixAdapter.html#server_info/2"},{"type":"module","title":"Bandit.Telemetry","doc":"The following telemetry spans are emitted by bandit\n\n## `[:bandit, :request, *]`\n\nRepresents Bandit handling a specific client HTTP request\n\nThis span is started by the following event:\n\n* `[:bandit, :request, :start]`\n\n    Represents the start of the span\n\n    This event contains the following measurements:\n\n    * `monotonic_time`: The time of this event, in `:native` units\n\n    This event contains the following metadata:\n\n    * `telemetry_span_context`: A unique identifier for this span\n    * `connection_telemetry_span_context`: The span context of the Thousand Island `:connection`\n      span which contains this request\n    * `conn`: The `Plug.Conn` representing this connection. Not present in cases where `error`\n      is also set and the nature of error is such that Bandit was unable to successfully build\n      the conn\n\nThis span is ended by the following event:\n\n* `[:bandit, :request, :stop]`\n\n    Represents the end of the span\n\n    This event contains the following measurements:\n\n    * `monotonic_time`: The time of this event, in `:native` units\n    * `duration`: The span duration, in `:native` units\n    * `req_header_end_time`: The time that header reading completed, in `:native` units\n    * `req_body_start_time`: The time that request body reading started, in `:native` units.\n    * `req_body_end_time`: The time that request body reading completed, in `:native` units\n    * `req_body_bytes`: The length of the request body, in octets\n    * `resp_start_time`: The time that the response started, in `:native` units\n    * `resp_end_time`: The time that the response completed, in `:native` units\n    * `resp_body_bytes`: The length of the response body, in octets. If the response is\n      compressed, this is the size of the compressed payload as sent on the wire\n    * `resp_uncompressed_body_bytes`: The length of the original, uncompressed body. Only\n      included for responses which are compressed\n    * `resp_compression_method`: The method of compression, as sent in the `Content-Encoding`\n      header of the response. Only included for responses which are compressed\n\n    This event contains the following metadata:\n\n    * `telemetry_span_context`: A unique identifier for this span\n    * `connection_telemetry_span_context`: The span context of the Thousand Island `:connection`\n      span which contains this request\n    * `conn`: The `Plug.Conn` representing this connection. Not present in cases where `error`\n      is also set and the nature of error is such that Bandit was unable to successfully build\n      the conn\n    * `error`: The error that caused the span to end, if it ended in error\n\nThe following events may be emitted within this span:\n\n* `[:bandit, :request, :exception]`\n\n    The request for this span ended unexpectedly\n\n    This event contains the following measurements:\n\n    * `monotonic_time`: The time of this event, in `:native` units\n\n    This event contains the following metadata:\n\n    * `telemetry_span_context`: A unique identifier for this span\n    * `connection_telemetry_span_context`: The span context of the Thousand Island `:connection`\n      span which contains this request\n    * `conn`: The `Plug.Conn` representing this connection. Not present in cases where `error`\n      is also set and the nature of error is such that Bandit was unable to successfully build\n      the conn\n    * `kind`: The kind of unexpected condition, typically `:exit`\n    * `exception`: The exception which caused this unexpected termination\n    * `stacktrace`: The stacktrace of the location which caused this unexpected termination\n\n## `[:bandit, :websocket, *]`\n\nRepresents Bandit handling a WebSocket connection\n\nThis span is started by the following event:\n\n* `[:bandit, :websocket, :start]`\n\n    Represents the start of the span\n\n    This event contains the following measurements:\n\n    * `monotonic_time`: The time of this event, in `:native` units\n    * `compress`: Details about the compression configuration for this connection\n\n    This event contains the following metadata:\n\n    * `telemetry_span_context`: A unique identifier for this span\n    * `connection_telemetry_span_context`: The span context of the Thousand Island `:connection`\n      span which contains this request\n\nThis span is ended by the following event:\n\n* `[:bandit, :websocket, :stop]`\n\n    Represents the end of the span\n\n    This event contains the following measurements:\n\n    * `monotonic_time`: The time of this event, in `:native` units\n    * `duration`: The span duration, in `:native` units\n    * `recv_text_frame_count`: The number of text frames received\n    * `recv_text_frame_bytes`: The total number of bytes received in the payload of text frames\n    * `recv_binary_frame_count`: The number of binary frames received\n    * `recv_binary_frame_bytes`: The total number of bytes received in the payload of binary frames\n    * `recv_ping_frame_count`: The number of ping frames received\n    * `recv_ping_frame_bytes`: The total number of bytes received in the payload of ping frames\n    * `recv_pong_frame_count`: The number of pong frames received\n    * `recv_pong_frame_bytes`: The total number of bytes received in the payload of pong frames\n    * `recv_connection_close_frame_count`: The number of connection close frames received\n    * `recv_connection_close_frame_bytes`: The total number of bytes received in the payload of connection close frames\n    * `recv_continuation_frame_count`: The number of continuation frames received\n    * `recv_continuation_frame_bytes`: The total number of bytes received in the payload of continuation frames\n    * `send_text_frame_count`: The number of text frames sent\n    * `send_text_frame_bytes`: The total number of bytes sent in the payload of text frames\n    * `send_binary_frame_count`: The number of binary frames sent\n    * `send_binary_frame_bytes`: The total number of bytes sent in the payload of binary frames\n    * `send_ping_frame_count`: The number of ping frames sent\n    * `send_ping_frame_bytes`: The total number of bytes sent in the payload of ping frames\n    * `send_pong_frame_count`: The number of pong frames sent\n    * `send_pong_frame_bytes`: The total number of bytes sent in the payload of pong frames\n    * `send_connection_close_frame_count`: The number of connection close frames sent\n    * `send_connection_close_frame_bytes`: The total number of bytes sent in the payload of connection close frames\n    * `send_continuation_frame_count`: The number of continuation frames sent\n    * `send_continuation_frame_bytes`: The total number of bytes sent in the payload of continuation frames\n\n    This event contains the following metadata:\n\n    * `telemetry_span_context`: A unique identifier for this span\n    * `origin_telemetry_span_context`: The span context of the Bandit `:request` span from which\n      this connection originated\n    * `connection_telemetry_span_context`: The span context of the Thousand Island `:connection`\n      span which contains this request\n    * `error`: The error that caused the span to end, if it ended in error","ref":"Bandit.Telemetry.html"},{"type":"function","title":"Bandit.Telemetry.monotonic_time/0","doc":"","ref":"Bandit.Telemetry.html#monotonic_time/0"},{"type":"function","title":"Bandit.Telemetry.span_exception/4","doc":"","ref":"Bandit.Telemetry.html#span_exception/4"},{"type":"opaque","title":"Bandit.Telemetry.t/0","doc":"","ref":"Bandit.Telemetry.html#t:t/0"},{"type":"exception","title":"Bandit.HTTP2.Errors.ConnectionError","doc":"","ref":"Bandit.HTTP2.Errors.ConnectionError.html"},{"type":"exception","title":"Bandit.HTTP2.Errors.StreamError","doc":"","ref":"Bandit.HTTP2.Errors.StreamError.html"},{"type":"exception","title":"Bandit.HTTPError","doc":"","ref":"Bandit.HTTPError.html"},{"type":"extras","title":"Changelog","doc":"","ref":"changelog.html"},{"type":"extras","title":"1.4.1 (27 Mar 2024) - Changelog","doc":"#","ref":"changelog.html#1-4-1-27-mar-2024"},{"type":"extras","title":"Changes - Changelog","doc":"* **BREAKING CHANGE** Move `log_protocol_errors` configuration option into\n  shared `http_options` top-level config (and apply it to HTTP/2 errors as well)\n* **BREAKING CHANGE** Remove `origin_telemetry_span_context` from WebSocket\n  telemetry events\n* **BREAKING CHANGE** Remove `stream_id` from HTTP/2 telemetry events\n* Add `conn` to the metadata of telemetry start events for HTTP requests\n* Stop sending WebSocket upgrade failure reasons to the client (they're still\n  logged)\n\n#","ref":"changelog.html#changes"},{"type":"extras","title":"Fixes - Changelog","doc":"* Return HTTP semantic errors to HTTP/2 clients as protocol errors instead of\n  internal errors","ref":"changelog.html#fixes"},{"type":"extras","title":"1.4.0 (26 Mar 2024) - Changelog","doc":"> [!WARNING]\n> **IMPORTANT** Phoenix users MUST upgrade to WebSockAdapter `0.5.6` or newer when\n> upgrading to Bandit `1.4.0` or newer as some internal module names have changed\n\n#","ref":"changelog.html#1-4-0-26-mar-2024"},{"type":"extras","title":"Enhancements - Changelog","doc":"* Complete refactor of HTTP/2. Improved process model is MUCH easier to\n  understand and yields about a 10% performance boost to HTTP/2 requests (#286 /\n  #307)\n* Substantial refactor of the HTTP/1 and HTTP/2 stacks to share a common code\n  path for much of their implementations, with the protocol-specific parts being\n  factored out to a minimal `Bandit.HTTPTransport` protocol internally, which\n  allows each protocol to define its own implementation for the minimal set of\n  things that are different between the two stacks (#297 / #329)\n\n#","ref":"changelog.html#enhancements"},{"type":"extras","title":"Changes - Changelog","doc":"* **BREAKING CHANGE** Move configuration options that are common between HTTP/1\n  and HTTP/2 stacks into a shared `http_options` top-level config\n* **BREAKING CHANGE** The HTTP/2 header size limit options have been deprecated,\n  and have been replaced with a single `max_header_block_size` option. The setting\n  defaults to 50k bytes, and refers to the size of the compressed header block\n  as sent on the wire (including any continuation frames)\n* **BREAKING CHANGE** Remove `req_line_bytes`, `req_header_bytes`, `resp_line_bytes` and\n  `resp_header_bytes` from HTTP/1 request telemetry measurements\n* **BREAKING CHANGE** Remove `status`, `method` and `request_target` from\n  telemetry metadata. All of this information can be obtained from the `conn`\n  struct attached to most telemetry events\n* **BREAKING CHANGE** Re-reading a body that has already been read returns `{:ok,\n  \"\", conn}` instead of raising a `Bandit.BodyAlreadyReadError`\n* **BREAKING CHANGE** Remove `Bandit.BodyAlreadyReadError`\n* **BREAKING CHANGE** Remove h2c support via Upgrade header. This was deprecated\n  in RFC9113 and never in widespread use. We continue to support h2c via prior\n  knowledge, which remains the only supported mechanism for h2c in RFC9113\n* Treat trailing bytes beyond the indicated content-length on HTTP/1 requests as\n  an error\n* Surface request body read timeouts on HTTP/1 requests as `{:more...}` tuples\n  and not errors\n* Socket sending errors are no longer surfaced on chunk sends in HTTP/1\n* We no longer log if processes that are linked to an HTTP/2 stream process\n  terminate unexpectedly. This has always been unspecified behaviour so is not\n  considered a breaking change\n* Calls of `Plug.Conn` functions for an HTTP/2 connection must now come from the\n  stream process; any other process will raise an error. Again, this has always\n  been unspecified behaviour\n* We now send an empty DATA frame for explicitly zero byte bodies instead of\n  optimizing to a HEADERS frame with end_stream set (we still do so for cases\n  such as 204/304 and HEAD requests)\n* We now send RST_STREAM frames if we complete a stream and the remote end is\n  still open. This optimizes cases where the client may still be sending a body\n  that we never consumed and don't care about\n* We no longer explicitly close the connection when we receive a GOAWAY frame","ref":"changelog.html#changes"},{"type":"extras","title":"1.3.0 (8 Mar 2024) - Changelog","doc":"#","ref":"changelog.html#1-3-0-8-mar-2024"},{"type":"extras","title":"Enhancements - Changelog","doc":"* Run an explicit garbage collection between every 'n' keepalive requests on the same HTTP/1.1 connection in order to keep reported (but not actual!) memory usage from growing over time. Add `gc_every_n_keepalive_requests` option to configure this (default value of\n  `5`). #322, thanks @ianko & @Nilsonn!)\n* Add `log_protocol_errors` option to optionally quell console logging of 4xx errors generated by Bandit. Defaults to `true` for now; may switch to `false` in the future based on adoption (#321, thanks @Stroemgren!)\n\n#","ref":"changelog.html#enhancements"},{"type":"extras","title":"Changes - Changelog","doc":"* Don't send a `transfer-encoding` header for 1xx or 204 responses (#317, thanks\n  @mwhitworth!)","ref":"changelog.html#changes"},{"type":"extras","title":"1.2.3 (23 Feb 2024) - Changelog","doc":"#","ref":"changelog.html#1-2-3-23-feb-2024"},{"type":"extras","title":"Changes - Changelog","doc":"* Log port number when listen fails (#312, thanks @jonatanklosko!)\n* Accept mixed-case keepalive directives (#308, thanks @gregors!)","ref":"changelog.html#changes"},{"type":"extras","title":"1.2.2 (16 Feb 2024) - Changelog","doc":"#","ref":"changelog.html#1-2-2-16-feb-2024"},{"type":"extras","title":"Changes - Changelog","doc":"* Reset Logger metadata on every request","ref":"changelog.html#changes"},{"type":"extras","title":"1.2.1 (12 Feb 2024) - Changelog","doc":"#","ref":"changelog.html#1-2-1-12-feb-2024"},{"type":"extras","title":"Changes - Changelog","doc":"* Disable logging of unknown messages received by an idle HTTP/1 handler to\n  avoid noise on long polling clients. This can be changed via the\n  `log_unknown_messages` http_1 option (#299)","ref":"changelog.html#changes"},{"type":"extras","title":"1.2.0 (31 Jan 2024) - Changelog","doc":"#","ref":"changelog.html#1-2-0-31-jan-2024"},{"type":"extras","title":"Enhancements - Changelog","doc":"* Automatically pull in `:otp_app` value in Bandit.PhoenixAdapter (thanks\n  @krns!)\n* Include response body metrics for HTTP/1 chunk responses\n\n#","ref":"changelog.html#enhancements"},{"type":"extras","title":"Fixes - Changelog","doc":"* Fix broken HTTP/1 inform/3 return value (thanks @wojtekmach!)\n* Maintain HTTP/1 read timeout after receiving unknown messages","ref":"changelog.html#fixes"},{"type":"extras","title":"1.1.3 (12 Jan 2024) - Changelog","doc":"#","ref":"changelog.html#1-1-3-12-jan-2024"},{"type":"extras","title":"Fixes - Changelog","doc":"* Do not send a fallback response if the plug has already sent one (#288 & #289, thanks @jclem!)\n\n#","ref":"changelog.html#fixes"},{"type":"extras","title":"Changes - Changelog","doc":"* Packagaing improvements (#283, thanks @wojtekmach!)","ref":"changelog.html#changes"},{"type":"extras","title":"1.1.2 (20 Dec 2023) - Changelog","doc":"#","ref":"changelog.html#1-1-2-20-dec-2023"},{"type":"extras","title":"Fixes - Changelog","doc":"* Fix support for proplist-style arguments (#277, thanks @jjcarstens!)\n* Speed up WebSocket framing (#272, thanks @crertel!)\n* Fix off-by-one error in HTTP2 sendfile (#269, thanks @OrangeDrangon!)\n* Improve mix file packaging (#266, thanks @patrickjaberg!)","ref":"changelog.html#fixes"},{"type":"extras","title":"1.1.1 (14 Nov 2023) - Changelog","doc":"#","ref":"changelog.html#1-1-1-14-nov-2023"},{"type":"extras","title":"Fixes - Changelog","doc":"* Do not advertise disabled protocols via ALPN (#263)","ref":"changelog.html#fixes"},{"type":"extras","title":"1.1.0 (2 Nov 2023) - Changelog","doc":"#","ref":"changelog.html#1-1-0-2-nov-2023"},{"type":"extras","title":"Changes - Changelog","doc":"* Messages sent to Bandit HTTP/1 handlers no longer intentionally crash the\n  handler process but are now logged in the same manner as messages sent to a\n  no-op GenServer (#259)\n* Messages regarding normal termination of monitored processes are no longer\n  handled by the WebSocket handler, but are now passed to the configured\n  `c:WebSock.handle_info/2` callback (#259)\n\n#","ref":"changelog.html#changes"},{"type":"extras","title":"Enhancements - Changelog","doc":"* Add support for `Phoenix.Endpoint.server_info/1` (now in Phoenix main; #258)\n* Add support for `:max_heap_size` option in WebSocket handler (introduced in\n  websock_adapter 0.5.5; #255, thanks @v0idpwn!)","ref":"changelog.html#enhancements"},{"type":"extras","title":"1.0.0 (18 Oct 2023) - Changelog","doc":"#","ref":"changelog.html#1-0-0-18-oct-2023"},{"type":"extras","title":"Changes - Changelog","doc":"* Remove internal tracking of remote `max_concurrent_streams` setting (#248)","ref":"changelog.html#changes"},{"type":"extras","title":"1.0.0-pre.18 (10 Oct 2023) - Changelog","doc":"#","ref":"changelog.html#1-0-0-pre-18-10-oct-2023"},{"type":"extras","title":"Fixes - Changelog","doc":"* Fix startup when plug module has not yet been loaded by the BEAM","ref":"changelog.html#fixes"},{"type":"extras","title":"1.0.0-pre.17 (9 Oct 2023) - Changelog","doc":"#","ref":"changelog.html#1-0-0-pre-17-9-oct-2023"},{"type":"extras","title":"Enhancements - Changelog","doc":"* Support function based plugs & improve startup analysis of plug configuration\n  (#236)\n* Improve keepalive support when Plug does not read request bodies (#244)\n* Improve logic around not sending bodies on HEAD requests (#242)\n\n#","ref":"changelog.html#enhancements"},{"type":"extras","title":"Changes - Changelog","doc":"* Internal refactor of WebSocket validation (#229)","ref":"changelog.html#changes"},{"type":"extras","title":"1.0.0-pre.16 (18 Sep 2023) - Changelog","doc":"#","ref":"changelog.html#1-0-0-pre-16-18-sep-2023"},{"type":"extras","title":"Changes - Changelog","doc":"* Use protocol default port in the event that no port is provided in host header (#228)\n\n#","ref":"changelog.html#changes"},{"type":"extras","title":"Fixes - Changelog","doc":"* Improve handling of iolist response bodies (#231, thanks @travelmassive!)","ref":"changelog.html#fixes"},{"type":"extras","title":"1.0.0-pre.15 (9 Sep 2023) - Changelog","doc":"#","ref":"changelog.html#1-0-0-pre-15-9-sep-2023"},{"type":"extras","title":"Fixes - Changelog","doc":"* Fix issue with setting remote IP at connection startup (#227, thanks @jimc64!)","ref":"changelog.html#fixes"},{"type":"extras","title":"1.0.0-pre.14 (28 Aug 2023) - Changelog","doc":"#","ref":"changelog.html#1-0-0-pre-14-28-aug-2023"},{"type":"extras","title":"Enhancements - Changelog","doc":"* Add `Bandit.PhoenixAdapter.bandit_pid/2` (#212)\n* Return errors to `Plug.Conn.Adapter.chunk/2` HTTP/1 calls (#216)\n\n#","ref":"changelog.html#enhancements"},{"type":"extras","title":"Changes - Changelog","doc":"* `Plug.Conn` function calls must come from the process on which `Plug.call/2` was called (#217, reverts #117)","ref":"changelog.html#changes"},{"type":"extras","title":"1.0.0-pre.13 (15 Aug 2023) - Changelog","doc":"#","ref":"changelog.html#1-0-0-pre-13-15-aug-2023"},{"type":"extras","title":"Enhancements - Changelog","doc":"* Add ability to send preamble frames when closing a WebSock connection (#211)","ref":"changelog.html#enhancements"},{"type":"extras","title":"1.0.0-pre.12 (12 Aug 2023) - Changelog","doc":"","ref":"changelog.html#1-0-0-pre-12-12-aug-2023"},{"type":"extras","title":"Fixes - Changelog","doc":"* Bump ThousandIsland to 1.0.0-pre.7 to fix leaking file descriptors on\n  `Plug.Conn.sendfile/5` calls (thanks @Hermanverschooten!)","ref":"changelog.html#fixes"},{"type":"extras","title":"1.0.0-pre.11 (11 Aug 2023) - Changelog","doc":"","ref":"changelog.html#1-0-0-pre-11-11-aug-2023"},{"type":"extras","title":"Changes - Changelog","doc":"* **BREAKING CHANGE** Move `conn` value in telemetry events from measurements to metadata","ref":"changelog.html#changes"},{"type":"extras","title":"Enhancements - Changelog","doc":"* Add `method`, `request_target` and `status` fields to telemetry metadata on HTTP stop events\n* Improve RFC compliance regarding cache-related headers on deflated responses (#207, thanks @tanguilp!)\n* Bump to Thousand Island `1.0.0-pre.6`\n* Doc improvements (particularly around implementation notes)\n* Typespec improvements (thanks @moogle19!)","ref":"changelog.html#enhancements"},{"type":"extras","title":"1.0.0-pre.10 (28 Jun 2023) - Changelog","doc":"","ref":"changelog.html#1-0-0-pre-10-28-jun-2023"},{"type":"extras","title":"Enhancements - Changelog","doc":"* Add support for `Plug.Conn.inform/3` on HTTP/1 connections (#180)\n* Add support for h2c upgrades (#186, thanks @alisinabh!)\n* Internal refactoring of HTTP/1 content-length encoded body reads (#184, #190,\n  thanks @asakura & @moogle19!)","ref":"changelog.html#enhancements"},{"type":"extras","title":"Changes - Changelog","doc":"* Bump Thousand Island to 1.0.0-pre.6 (gaining support for suspend/resume API)\n* Drop Elixir 1.12 as a supported target (it should continue to work, but is no\n  longer covered by CI)","ref":"changelog.html#changes"},{"type":"extras","title":"Fixes - Changelog","doc":"* Fix crash when Plug used `Plug.Conn.get_peer_data/1` function on HTTP/1\n  connections (#170, thanks @moogle19!)\n* Fix port behaviour when connecting over unix socket (#176, thanks @asakura\n  & @ibarchenkov!)","ref":"changelog.html#fixes"},{"type":"extras","title":"1.0.0-pre.9 (16 Jun 2023) - Changelog","doc":"","ref":"changelog.html#1-0-0-pre-9-16-jun-2023"},{"type":"extras","title":"Changes - Changelog","doc":"* Use new ThousandIsland APIs for socket info (#167, thanks @asakura!)","ref":"changelog.html#changes"},{"type":"extras","title":"Fixes - Changelog","doc":"* Handle nil connection close reason when closing a WebSocket","ref":"changelog.html#fixes"},{"type":"extras","title":"1.0.0-pre.8 (15 Jun 2023) - Changelog","doc":"","ref":"changelog.html#1-0-0-pre-8-15-jun-2023"},{"type":"extras","title":"Fixes - Changelog","doc":"* Further improve logging on WebSocket upgrade errors (#149)","ref":"changelog.html#fixes"},{"type":"extras","title":"1.0.0-pre.7 (14 Jun 2023) - Changelog","doc":"","ref":"changelog.html#1-0-0-pre-7-14-jun-2023"},{"type":"extras","title":"Enhancements - Changelog","doc":"* Refactor HTTP/1 read routines (#158 & #166, thanks @asakura!)\n* Improve logging on WebSocket upgrade errors (#149)","ref":"changelog.html#enhancements"},{"type":"extras","title":"Changes - Changelog","doc":"* Override any content-length headers that may have been set by Plug (#165)\n* Send content-length on HTTP/2 responses where appropriate (#165)","ref":"changelog.html#changes"},{"type":"extras","title":"Fixes - Changelog","doc":"* Send correct content-length header when sending deflated response (#151)\n* Do not attempt to deflate if Plug sends a content-encoding header (#165)\n* Improve corner case handling of content-length request header (#163, thanks\n  @ryanwinchester!)\n* Handle case where ThousandIsland returns error tuples on some helper routines\n  (#162)","ref":"changelog.html#fixes"},{"type":"extras","title":"1.0.0-pre.6 (8 Jun 2023) - Changelog","doc":"#","ref":"changelog.html#1-0-0-pre-6-8-jun-2023"},{"type":"extras","title":"Changes - Changelog","doc":"* Always use the declaed scheme if declared in a request-line or `:scheme`\n  pseudo-header (#159)\n* Internal tidying (thanks @asakura!)","ref":"changelog.html#changes"},{"type":"extras","title":"1.0.0-pre.5 (2 Jun 2023) - Changelog","doc":"#","ref":"changelog.html#1-0-0-pre-5-2-jun-2023"},{"type":"extras","title":"Enhancements - Changelog","doc":"* Total overhaul of typespecs throughout the library (thanks @asakura!)","ref":"changelog.html#enhancements"},{"type":"extras","title":"1.0.0-pre.4 (23 May 2023) - Changelog","doc":"#","ref":"changelog.html#1-0-0-pre-4-23-may-2023"},{"type":"extras","title":"Enhancements - Changelog","doc":"* Performance / correctness improvements to header length validation (#143,\n  thanks @moogle19!)\n* Performance improvements to host header port parsing (#145 & #147, thanks\n  @ryanwinchester!)\n* Improve WebSocket upgrade failure error messages to aid in diagnosis (#152)\n\n#","ref":"changelog.html#enhancements"},{"type":"extras","title":"Changes - Changelog","doc":"* Consolidate credo config (#146, thanks @ryanwinchester!)\n\n#","ref":"changelog.html#changes"},{"type":"extras","title":"Fixes - Changelog","doc":"* Fix error in suggested version dependencies during 1.0-pre series (#142,\n  thanks @cvkmohan!)","ref":"changelog.html#fixes"},{"type":"extras","title":"1.0.0-pre.3 (3 May 2023) - Changelog","doc":"#","ref":"changelog.html#1-0-0-pre-3-3-may-2023"},{"type":"extras","title":"Enhancements - Changelog","doc":"* Respect read timeout for HTTP/1 keepalives (#140)\n* Support Websock 0.5.1, including support for optional `c:WebSock.terminate/2`\n  (#131)\n\n#","ref":"changelog.html#enhancements"},{"type":"extras","title":"Changes - Changelog","doc":"* Use Req instead of Finch in tests (#137)\n* Improve a few corner cases in tests (#136)","ref":"changelog.html#changes"},{"type":"extras","title":"1.0.0-pre.2 (24 Apr 2023) - Changelog","doc":"#","ref":"changelog.html#1-0-0-pre-2-24-apr-2023"},{"type":"extras","title":"Fixes - Changelog","doc":"* Don't require transport_options to be a keyword list (#130, thanks @justinludwig!)","ref":"changelog.html#fixes"},{"type":"extras","title":"1.0.0-pre.1 (21 Apr 2023) - Changelog","doc":"#","ref":"changelog.html#1-0-0-pre-1-21-apr-2023"},{"type":"extras","title":"Changes - Changelog","doc":"* Update Thousand Island dependency to 1.0-pre\n\n# Changelog for 0.7.x","ref":"changelog.html#changes"},{"type":"extras","title":"0.7.7 (11 Apr 2023) - Changelog","doc":"#","ref":"changelog.html#0-7-7-11-apr-2023"},{"type":"extras","title":"Changes - Changelog","doc":"* Bandit will now raise an error at startup if no plug is specified in config\n  (thanks @moogle19!)\n\n#","ref":"changelog.html#changes"},{"type":"extras","title":"Fixes - Changelog","doc":"* Fix crash at startup when using `otp_app` option (thanks @moogle19!)\n* Minor doc formatting fixes","ref":"changelog.html#fixes"},{"type":"extras","title":"0.7.6 (9 Apr 2023) - Changelog","doc":"#","ref":"changelog.html#0-7-6-9-apr-2023"},{"type":"extras","title":"Changes - Changelog","doc":"* **BREAKING CHANGE** Rename top-level `options` field to `thousand_island_options`\n* **BREAKING CHANGE** Rename `deflate_opts` to `deflate_options` where used\n* Massive overhaul of documentation to use types where possible\n* Bandit now uses a term of the form `{Bandit, ref()}` for `id` in our child spec\n* Bumped to Thousand Island 0.6.7. `num_connections` is now 16384 by default\n\n#","ref":"changelog.html#changes"},{"type":"extras","title":"Enhancements - Changelog","doc":"* Added top level support for the following convenience parameters:\n  * `port` can now be set at the top level of your configuration\n  * `ip` can now be set at the top level of your configuration\n  * `keyfile` and `certfile` can now be set at the top level of your configuration\n* Transport options are now validated by `Plug.SSL.configure/1` when starting\n  an HTTPS server\n* Rely on Thousand Island to validate options specified in `thousand_island_options`. This should avoid cases like #125 in the future.","ref":"changelog.html#enhancements"},{"type":"extras","title":"0.7.5 (4 Apr 2023) - Changelog","doc":"#","ref":"changelog.html#0-7-5-4-apr-2023"},{"type":"extras","title":"Changes - Changelog","doc":"* Drop explicit support for Elixir 1.11 since we no longer test it in CI (should\n  still work, just that it's now at-your-own-risk)\n* Add logo to ex_doc and README\n\n#","ref":"changelog.html#changes"},{"type":"extras","title":"Fixes - Changelog","doc":"* Allow access to Thousand Island's underlying `shutdown_timeout` option\n* Fix test errors that cropped up in OTP 26","ref":"changelog.html#fixes"},{"type":"extras","title":"0.7.4 (27 Mar 2023) - Changelog","doc":"#","ref":"changelog.html#0-7-4-27-mar-2023"},{"type":"extras","title":"Changes - Changelog","doc":"* Calling `Plug.Conn` adapter functions for HTTP/2 based requests are no longer\n  restricted to being called from the process which called `c:Plug.call/2`\n\n#","ref":"changelog.html#changes"},{"type":"extras","title":"Enhancements - Changelog","doc":"* Added `startup_log` to control whether / how Bandit logs the bound host & port\n  at startup (Thanks @danschultzer)\n* Improved logging when the configured port is in use at startup (Thanks\n  @danschultzer)\n* Update to Thousand Island 0.6.5","ref":"changelog.html#enhancements"},{"type":"extras","title":"0.7.3 (20 Mar 2023) - Changelog","doc":"#","ref":"changelog.html#0-7-3-20-mar-2023"},{"type":"extras","title":"Enhancements - Changelog","doc":"* Added advanced `handler_module` configuration option to `options`\n\n#","ref":"changelog.html#enhancements"},{"type":"extras","title":"Fixes - Changelog","doc":"* Support returning `x-gzip` as negotiated `content-encoding` (previously would\n  negotiate a request for `x-gzip` as `gzip`)","ref":"changelog.html#fixes"},{"type":"extras","title":"0.7.2 (18 Mar 2023) - Changelog","doc":"#","ref":"changelog.html#0-7-2-18-mar-2023"},{"type":"extras","title":"Enhancements - Changelog","doc":"* Added HTTP compression via 'Content-Encoding' negotiation, enabled by default.\n  Configuration is available; see [Bandit\n  docs](https://hexdocs.pm/bandit/Bandit.html#module-config-options) for details\n\n#","ref":"changelog.html#enhancements"},{"type":"extras","title":"Changes - Changelog","doc":"* Minor refactor of internal HTTP/2 plumbing. No user visible changes","ref":"changelog.html#changes"},{"type":"extras","title":"0.7.1 (17 Mar 2023) - Changelog","doc":"#","ref":"changelog.html#0-7-1-17-mar-2023"},{"type":"extras","title":"Changes - Changelog","doc":"* Update documentation & messaging to refer to RFC911x RFCs where appropriate\n* Validate top-level config options at startup\n* Revise Phoenix adapter to support new config options\n* Doc updates","ref":"changelog.html#changes"},{"type":"extras","title":"0.7.0 (17 Mar 2023) - Changelog","doc":"#","ref":"changelog.html#0-7-0-17-mar-2023"},{"type":"extras","title":"Enhancements - Changelog","doc":"* Add configuration points for various parameters within the HTTP/1, HTTP/2 and\n  WebSocket stacks. See [Bandit\n  docs](https://hexdocs.pm/bandit/Bandit.html#module-config-options) for details\n\n# Changelog for 0.6.x","ref":"changelog.html#enhancements"},{"type":"extras","title":"0.6.11 (17 Mar 2023) - Changelog","doc":"#","ref":"changelog.html#0-6-11-17-mar-2023"},{"type":"extras","title":"Changes - Changelog","doc":"* Modified telemetry event payloads to match the conventions espoused by\n  `:telemetry.span/3`\n* Default shutdown timeout is now 15s (up from 5s)\n\n#","ref":"changelog.html#changes"},{"type":"extras","title":"Enhancements - Changelog","doc":"* Update to Thosuand Island 0.6.4 (from 0.6.2)","ref":"changelog.html#enhancements"},{"type":"extras","title":"0.6.10 (10 Mar 2023) - Changelog","doc":"#","ref":"changelog.html#0-6-10-10-mar-2023"},{"type":"extras","title":"Enhancements - Changelog","doc":"* Support explicit setting of WebSocket close codes & reasons as added in WebSock\n0.5.0","ref":"changelog.html#enhancements"},{"type":"extras","title":"0.6.9 (20 Feb 2023) - Changelog","doc":"#","ref":"changelog.html#0-6-9-20-feb-2023"},{"type":"extras","title":"Enhancements - Changelog","doc":"* Add comprehensive Telemetry support within Bandit, as documented in the\n  `Bandit.Telemetry` module\n* Update our ThousandIsland dependnecy to pull in Thousand Island's newly\n  updated Telemetry support as documented in the `ThousandIsland.Telemetry`\n  module\n* Fix parsing of host / request headers which contain IPv6 addresses (#97).\n  Thanks @derekkraan!\n\n# Changes\n\n* Use Plug's list of response code reason phrases (#96). Thanks @jclem!\n* Minor doc updates","ref":"changelog.html#enhancements"},{"type":"extras","title":"0.6.8 (31 Jan 2023) - Changelog","doc":"#","ref":"changelog.html#0-6-8-31-jan-2023"},{"type":"extras","title":"Changes - Changelog","doc":"* Close WebSocket connections with a code of 1000 (instead of 1001) when\n  shutting down the server (#89)\n* Use 100 acceptor processes by default (instead of 10)\n* Improvements to make WebSocket frame masking faster","ref":"changelog.html#changes"},{"type":"extras","title":"0.6.7 (17 Jan 2023) - Changelog","doc":"#","ref":"changelog.html#0-6-7-17-jan-2023"},{"type":"extras","title":"Enhancements - Changelog","doc":"* Remove logging entirely when client connections do not contain a valid protocol\n* Refactor WebSocket support for about a 20% performance lift\n\n#","ref":"changelog.html#enhancements"},{"type":"extras","title":"Bug Fixes - Changelog","doc":"* Add `nodelay` option to test suite to fix artificially slow WebSocket perf tests","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"0.6.6 (11 Jan 2023) - Changelog","doc":"#","ref":"changelog.html#0-6-6-11-jan-2023"},{"type":"extras","title":"Enhancements - Changelog","doc":"* Log useful message when a TLS connection is made to plaintext server (#74)","ref":"changelog.html#enhancements"},{"type":"extras","title":"0.6.5 (10 Jan 2023) - Changelog","doc":"#","ref":"changelog.html#0-6-5-10-jan-2023"},{"type":"extras","title":"Enhancements - Changelog","doc":"* Update Thousand Island to 0.5.15 (quiets logging in timeout cases)\n* Quiet logging in when client connections do not contain a valid protocol\n* Refactor HTTP/1 for about a 20% performance lift\n* Add WebSocket support to CI benchmark workflow\n* Doc updates\n\n#","ref":"changelog.html#enhancements"},{"type":"extras","title":"Bug Fixes - Changelog","doc":"* Allow multiple instances of Bandit to be started in the same node (#75)\n* Improve error handling in HTTP/1 when protocol errors are encountered (#74)","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"README","doc":"![Bandit](https://github.com/mtrudel/bandit/raw/main/assets/readme_logo.png#gh-light-mode-only)\n![Bandit](https://github.com/mtrudel/bandit/raw/main/assets/readme_logo-darkmode.png#gh-dark-mode-only)\n\n[![Build Status](https://github.com/mtrudel/bandit/workflows/Elixir%20CI/badge.svg)](https://github.com/mtrudel/bandit/actions)\n[![Docs](https://img.shields.io/badge/api-docs-green.svg?style=flat)](https://hexdocs.pm/bandit)\n[![Hex.pm](https://img.shields.io/hexpm/v/bandit.svg?style=flat&color=blue)](https://hex.pm/packages/bandit)\n\nBandit is an HTTP server for Plug and WebSock apps.\n\nBandit is written entirely in Elixir and is built atop [Thousand\nIsland](https://github.com/mtrudel/thousand_island). It can serve HTTP/1.x,\nHTTP/2 and WebSocket clients over both HTTP and HTTPS. It is written with\ncorrectness, clarity & performance as fundamental goals.\n\nIn [ongoing automated performance\ntests](https://github.com/mtrudel/bandit/actions/runs/4449308920),\nBandit's HTTP/1.x engine is up to 4x faster than Cowboy depending on the number of concurrent\nrequests. When comparing HTTP/2 performance, Bandit is up to 1.5x faster than Cowboy. This is\npossible because Bandit has been built from the ground up for use with Plug applications; this\nfocus pays dividends in both performance and also in the approachability of the code base.\n\nBandit also emphasizes correctness. Its HTTP/2 implementation scores 100% on the\n[h2spec](https://github.com/summerwind/h2spec) suite in strict mode, and its\nWebSocket implementation scores 100% on the\n[Autobahn](https://github.com/crossbario/autobahn-testsuite) test suite, both of\nwhich run as part of Bandit's comprehensive CI suite. Extensive unit test,\ncredo, dialyzer, and performance regression test coverage round out a test suite\nthat ensures that Bandit is and will remain a platform you can count on.\n\nLastly, Bandit exists to demystify the lower layers of infrastructure code. In a world where\nThe New Thing is nearly always adding abstraction on top of abstraction, it's important to have\nfoundational work that is approachable & understandable by users above it in the stack.","ref":"readme.html"},{"type":"extras","title":"Project Goals - README","doc":"* Implement comprehensive support for HTTP/1.0 through HTTP/2 & WebSockets (and\n  beyond) backed by obsessive RFC literacy and automated conformance testing\n* Aim for minimal internal policy and HTTP-level configuration. Delegate to Plug & WebSock as much as\n  possible, and only interpret requests to the extent necessary to safely manage a connection\n  & fulfill the requirements of safely supporting protocol correctness\n* Prioritize (in order): correctness, clarity, performance. Seek to remove the mystery of\n  infrastructure code by being approachable and easy to understand\n* Along with our companion library [Thousand\n  Island](https://github.com/mtrudel/thousand_island), become the go-to HTTP\n  & low-level networking stack of choice for the Elixir community by being\n  reliable, efficient, and approachable","ref":"readme.html#project-goals"},{"type":"extras","title":"Project Status - README","doc":"* Complete support for [Phoenix](https://github.com/phoenixframework/phoenix) applications (WebSocket\n  support requires Phoenix 1.7+)\n* Complete support of the [Plug API](https://github.com/elixir-plug/plug)\n* Complete support of the [WebSock API](https://github.com/phoenixframework/websock)\n* Complete server support for HTTP/1.x as defined in [RFC\n  9112](https://datatracker.ietf.org/doc/html/rfc9112) & [RFC\n  9110](https://datatracker.ietf.org/doc/html/rfc9110)\n* Complete server support for HTTP/2 as defined in [RFC\n  9113](https://datatracker.ietf.org/doc/html/rfc9113) & [RFC\n  9110](https://datatracker.ietf.org/doc/html/rfc9110), comprehensively covered\n  by automated [h2spec](https://github.com/summerwind/h2spec) conformance testing\n* Support for HTTP content encoding compression on both HTTP/1.x and HTTP/2.\n  gzip and deflate methods are supported per\n  [RFC9110§8.4.1.{2,3}](https://www.rfc-editor.org/rfc/rfc9110.html#section-8.4.1.2)\n* Complete server support for WebSockets as defined in [RFC\n  6455](https://datatracker.ietf.org/doc/html/rfc6455), comprehensively covered by automated\n  [Autobahn](https://github.com/crossbario/autobahn-testsuite) conformance testing. Per-message\n  compression as defined in [RFC 7692](https://datatracker.ietf.org/doc/html/rfc7692) is also\n  supported\n* Extremely scalable and performant client handling at a rate up to 4x that of Cowboy for the same\n  workload with as-good-or-better memory use\n\nAny Phoenix or Plug app should work with Bandit as a drop-in replacement for\nCowboy; exceptions to this are errors (if you find one, please [file an\nissue!](https://github.com/mtrudel/bandit/issues)).\n\n<!-- MDOC -->","ref":"readme.html#project-status"},{"type":"extras","title":"Using Bandit With Phoenix - README","doc":"Bandit fully supports Phoenix. Phoenix applications which use WebSockets for\nfeatures such as Channels or LiveView require Phoenix 1.7 or later.\n\nUsing Bandit to host your Phoenix application couldn't be simpler:\n\n1. Add Bandit as a dependency in your Phoenix application's `mix.exs`:\n\n    ```elixir\n    {:bandit, \"~> 1.0\"}\n    ```\n2. Add the following `adapter:` line to your endpoint configuration in `config/config.exs`, as in the following example:\n\n     ```elixir\n     # config/config.exs\n\n     config :your_app, YourAppWeb.Endpoint,\n       adapter: Bandit.PhoenixAdapter, # <---- ADD THIS LINE\n       url: [host: \"localhost\"],\n       render_errors: ...\n     ```\n3. That's it! **You should now see messages at startup indicating that Phoenix is\n   using Bandit to serve your endpoint**, and everything should 'just work'. Note\n   that if you have set any exotic configuration options within your endpoint,\n   you may need to update that configuration to work with Bandit; see the\n   [Bandit.PhoenixAdapter](https://hexdocs.pm/bandit/Bandit.PhoenixAdapter.html)\n   documentation for more information.","ref":"readme.html#using-bandit-with-phoenix"},{"type":"extras","title":"Using Bandit With Plug Applications - README","doc":"Using Bandit to host your own Plug is very straightforward. Assuming you have\na Plug module implemented already, you can host it within Bandit by adding\nsomething similar to the following to your application's `Application.start/2`\nfunction:\n\n```elixir\n# lib/my_app/application.ex\n\ndefmodule MyApp.Application do\n  use Application\n\n  def start(_type, _args) do\n    children = [\n      {Bandit, plug: MyApp.MyPlug}\n    ]\n\n    opts = [strategy: :one_for_one, name: MyApp.Supervisor]\n    Supervisor.start_link(children, opts)\n  end\nend\n```\n\nFor less formal usage, you can also start Bandit using the same configuration\noptions via the `Bandit.start_link/1` function:\n\n```elixir\n# Start an http server on the default port 4000, serving MyApp.MyPlug\nBandit.start_link(plug: MyPlug)\n```","ref":"readme.html#using-bandit-with-plug-applications"},{"type":"extras","title":"Configuration - README","doc":"A number of options are defined when starting a server. The complete list is\ndefined by the [`t:Bandit.options/0`](https://hexdocs.pm/bandit/Bandit.html#summary) type.","ref":"readme.html#configuration"},{"type":"extras","title":"Setting up an HTTPS Server - README","doc":"By far the most common stumbling block encountered when setting up an HTTPS\nserver involves configuring key and certificate data. Bandit is comparatively\neasy to set up in this regard, with a working example looking similar to the\nfollowing:\n\n```elixir\n# lib/my_app/application.ex\n\ndefmodule MyApp.Application do\n  use Application\n\n  def start(_type, _args) do\n    children = [\n      {Bandit,\n       plug: MyApp.MyPlug,\n       scheme: :https,\n       certfile: \"/absolute/path/to/cert.pem\",\n       keyfile: \"/absolute/path/to/key.pem\"}\n    ]\n\n    opts = [strategy: :one_for_one, name: MyApp.Supervisor]\n    Supervisor.start_link(children, opts)\n  end\nend\n```","ref":"readme.html#setting-up-an-https-server"},{"type":"extras","title":"WebSocket Support - README","doc":"If you're using Bandit to run a Phoenix application as suggested above, there is\nnothing more for you to do; WebSocket support will 'just work'.\n\nIf you wish to interact with WebSockets at a more fundamental level, the\n[WebSock](https://hexdocs.pm/websock/WebSock.html) and\n[WebSockAdapter](https://hexdocs.pm/websock_adapter/WebSockAdapter.html) libraries\nprovides a generic abstraction for WebSockets (very similar to how Plug is\na generic abstraction on top of HTTP). Bandit fully supports all aspects of\nthese libraries.\n\n<!-- MDOC -->","ref":"readme.html#websocket-support"},{"type":"extras","title":"Implementation Details - README","doc":"Bandit primarily consists of three protocol-specific implementations, one each\nfor [HTTP/1][], [HTTP/2][] and [WebSockets][]. Each of these implementations is\nlargely distinct from one another, and is described in its own README linked\nabove.\n\nIf you're just taking a casual look at Bandit or trying to understand how an\nHTTP server works, the [HTTP/1][] implementation is likely the best place to\nstart exploring.\n\n[HTTP/1]: lib/bandit/http1/README.md\n[HTTP/2]: lib/bandit/http2/README.md\n[WebSockets]: lib/bandit/websocket/README.md","ref":"readme.html#implementation-details"},{"type":"extras","title":"Contributing - README","doc":"Contributions to Bandit are very much welcome! Before undertaking any substantial work, please\nopen an issue on the project to discuss ideas and planned approaches so we can ensure we keep\nprogress moving in the same direction.\n\nAll contributors must agree and adhere to the project's [Code of\nConduct](https://github.com/mtrudel/bandit/blob/main/CODE_OF_CONDUCT.md).\n\nSecurity disclosures should be handled per Bandit's published [security policy](https://github.com/mtrudel/bandit/blob/main/SECURITY.md).","ref":"readme.html#contributing"},{"type":"extras","title":"Installation - README","doc":"Bandit is [available in Hex](https://hex.pm/docs/publish). The package can be installed\nby adding `bandit` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:bandit, \"~> 1.0\"}\n  ]\nend\n```\n\nDocumentation can be found at [https://hexdocs.pm/bandit](https://hexdocs.pm/bandit).\n\n# License\n\nMIT","ref":"readme.html#installation"},{"type":"extras","title":"HTTP/1 Implementation Notes","doc":"# HTTP/1 Handler\n\nIncluded in this folder is a complete `ThousandIsland.Handler` based implementation of HTTP/1.x as\ndefined in [RFC 9112](https://datatracker.ietf.org/doc/rfc9112).","ref":"HTTP1_README.md.html"},{"type":"extras","title":"Process model - HTTP/1 Implementation Notes","doc":"Within a Bandit server, an HTTP/1 connection is modeled as a single process.\nThis process is tied to the lifecycle of the underlying TCP connection; in the\ncase of an HTTP client which makes use of HTTP's keep-alive feature to make\nmultiple requests on the same connection, all of these requests will be serviced\nby this same process.\n\nThe execution model to handle a given request is quite straightforward: the\nunderlying [Thousand Island](https://github.com/mtrudel/thousand_island) library\nwill call `Bandit.HTTP1.Handler.handle_data/3`, which will then construct a\n`Bandit.HTTP1.Socket` struct that conforms to the `Bandit.HTTPTransport`\nprotocol. It will then call `Bandit.Pipeline.run/3`, which will go through the\nprocess of reading the request (by calling functions on the\n`Bandit.HTTPTransport` protocol), and constructing a `Plug.Conn` structure to\nrepresent the request and subsequently pass it to the configured `Plug` module.\n\n# Testing\n\nAll of this is exhaustively tested. Tests are located in `request_test.exs`, and\nare broadly either concerned with testing network-facing aspects of the\nimplementation (ie: how well Bandit satisfies the relevant RFCs) or the Plug-facing\naspects of the implementation.\n\nUnfortunately, there is no HTTP/1 equivalent to the external h2spec test suite.","ref":"HTTP1_README.md.html#process-model"},{"type":"extras","title":"HTTP/2 Implementation Notes","doc":"# HTTP/2 Handler\n\nIncluded in this folder is a complete `ThousandIsland.Handler` based implementation of HTTP/2 as\ndefined in [RFC 9110](https://datatracker.ietf.org/doc/rfc9110) & [RFC\n9113](https://datatracker.ietf.org/doc/rfc9113)","ref":"HTTP2_README.md.html"},{"type":"extras","title":"Process model - HTTP/2 Implementation Notes","doc":"Within a Bandit server, an HTTP/2 connection is modeled as a set of processes:\n\n* 1 process per connection, a `Bandit.HTTP2.Handler` module implementing the\n  `ThousandIsland.Handler` behaviour, and;\n* 1 process per stream (i.e.: per HTTP request) within the connection, implemented as\n  a `Bandit.HTTP2.StreamProcess` process\n\nEach of these processes model the majority of their state via a\n`Bandit.HTTP2.Connection` & `Bandit.HTTP2.Stream` struct, respectively.\n\nThe lifetimes of these processes correspond to their role; a connection process lives for as long\nas a client is connected, and a stream process lives only as long as is required to process\na single stream request within a connection.\n\nConnection processes are the 'root' of each connection's process group, and are supervised by\nThousand Island in the same manner that `ThousandIsland.Handler` processes are usually supervised\n(see the [project README](https://github.com/mtrudel/thousand_island) for details).\n\nStream processes are not supervised by design. The connection process starts new\nstream processes as required, via a standard `start_link`\ncall, and manages the termination of the resultant linked stream processes by\nhandling `{:EXIT,...}` messages as described in the Elixir documentation. Each\nstream process stays alive long enough to fully model an HTTP/2 stream,\nbeginning its life in the `:init` state and ending it in the `:closed` state (or\nelse by a stream or connection error being raised). This approach is aligned\nwith the realities of the HTTP/2 model, insofar as if a connection process\nterminates there is no reason to keep its constituent stream processes around,\nand if a stream process dies the connection should be able to handle this\nwithout itself terminating. It also means that our process model is very\nlightweight - there is no extra supervision overhead present because no such\nsupervision is required for the system to function in the desired way.","ref":"HTTP2_README.md.html#process-model"},{"type":"extras","title":"Reading client data - HTTP/2 Implementation Notes","doc":"The overall structure of the implementation is managed by the `Bandit.HTTP2.Handler` module, and\nlooks like the following:\n\n1. Bytes are asynchronously received from ThousandIsland via the\n   `Bandit.HTTP2.Handler.handle_data/3` function\n2. Frames are parsed from these bytes by calling the `Bandit.HTTP2.Frame.deserialize/2`\n   function. If successful, the parsed frame(s) are returned. We retain any unparsed bytes in\n   a buffer in order to attempt parsing them upon receipt of subsequent data from the client\n3. Parsed frames are passed into the `Bandit.HTTP2.Connection` module along with a struct of\n   same module. Frames are processed via the `Bandit.HTTP2.Connection.handle_frame/3` function.\n   Connection-level frames are handled within the `Bandit.HTTP2.Connection`\n   struct, and stream-level frames are passed along to the corresponding stream\n   process, which is wholly responsible for managing all aspects of a stream's\n   state (which is tracked via the `Bandit.HTTP2.Stream` struct). The one\n   exception to this is the handling of frames sent to streams which have\n   already been closed (and whose corresponding processes have thus terminated).\n   Any such frames are discarded without effect.\n4. This process is repeated every time we receive data from the client until the\n   `Bandit.HTTP2.Connection` module indicates that the connection should be closed, either\n   normally or due to error. Note that frame deserialization may end up returning a connection\n   error if the parsed frames fail specific criteria (generally, the frame parsing modules are\n   responsible for identifying errors as described in [section\n   6](https://datatracker.ietf.org/doc/html/rfc9113#section-6) of RFC 9113). In these cases, the\n   failure is passed through to the connection module for processing in order to coordinate an\n   orderly shutdown or client notification as appropriate","ref":"HTTP2_README.md.html#reading-client-data"},{"type":"extras","title":"Processing requests - HTTP/2 Implementation Notes","doc":"The state of a particular stream are contained within a `Bandit.HTTP2.Stream`\nstruct, maintained within a `Bandit.HTTP2.StreamProcess` process. As part of the\nstream's lifecycle, the server's configured Plug is called, with an instance of\nthe `Bandit.Adapter` struct being used to interface with the Plug. There\nis a separation of concerns between the aspect of HTTP semantics managed by\n`Bandit.Adapter` (roughly, those concerns laid out in\n[RFC9110](https://datatracker.ietf.org/doc/html/rfc9110)) and the more\ntransport-specific HTTP/2 concerns managed by `Bandit.HTTP2.Stream` (roughly the\nconcerns specified in [RFC9113](https://datatracker.ietf.org/doc/html/rfc9113)).\n\n# Testing\n\nAll of this is exhaustively tested. Tests are broken up primarily into `protocol_test.exs`, which\nis concerned with aspects of the implementation relating to protocol conformance and\nclient-facing concerns, while `plug_test.exs` is concerned with aspects of the implementation\nhaving to do with the Plug API and application-facing concerns. There are also more\nunit-style tests covering frame serialization and deserialization.\n\nIn addition, the `h2spec` conformance suite is run via a `System` wrapper & executes the entirety\nof the suite (in strict mode) against a running Bandit server.","ref":"HTTP2_README.md.html#processing-requests"},{"type":"extras","title":"Limitations and Assumptions - HTTP/2 Implementation Notes","doc":"Some limitations and assumptions of this implementation:\n\n* This handler assumes that the HTTP/2 connection preface has already been consumed from the\n  client. The `Bandit.InitialHandler` module uses this preface to discriminate between various\n  HTTP versions when determining which handler to use\n* Priority frames are parsed and validated, but do not induce any action on the part of the\n  server. There is no priority assigned to respective streams in terms of processing; all streams\n  are run in parallel as soon as they arrive\n* While flow control is completely implemented here, the specific values used for upload flow\n  control (that is, the end that we control) are fixed. Specifically, we attempt to maintain\n  fairly large windows in order to not restrict client uploads (we 'slow-start' window changes\n  upon receipt of first byte, mostly to retain parity between connection and stream window\n  management since connection windows cannot be changed via settings). The majority of flow\n  control logic has been encapsulated in the `Bandit.HTTP2.FlowControl` module should future\n  refinement be required","ref":"HTTP2_README.md.html#limitations-and-assumptions"},{"type":"extras","title":"WebSocket Implementation Notes","doc":"# WebSocket Handler\n\nIncluded in this folder is a complete `ThousandIsland.Handler` based implementation of WebSockets\nas defined in [RFC 6455](https://datatracker.ietf.org/doc/rfc6455).","ref":"WebSocket_README.md.html"},{"type":"extras","title":"Upgrade mechanism - WebSocket Implementation Notes","doc":"A good overview of this process is contained in this [ElixirConf EU\ntalk](https://www.youtube.com/watch?v=usKLrYl4zlY).\n\nUpgrading an HTTP connection to a WebSocket connection is coordinated by code\ncontained within several libraries, including Bandit,\n[WebSockAdapter](https://github.com/phoenixframework/websock_adapter), and\n[Plug](https://github.com/elixir-plug/plug). \n\nThe HTTP request containing the upgrade request is first passed to the user's\napplication as a standard Plug call. After inspecting the request and deeming it\na suitable upgrade candidate (via whatever policy the application dictates), the\nuser indicates a desire to upgrade the connection to a WebSocket by calling\n`WebSockAdapter.upgrade/4`, which checks that the request is a valid WebSocket\nupgrade request, and then calls `Plug.Conn.upgrade_adapter/3` to signal to\nBandit that the connection should be upgraded at the conclusion of the request.\nAt the conclusion of the `Plug.call/2` callback, `Bandit.Pipeline` will then\nattempt to upgrade the underlying connection. As part of this upgrade process,\n`Bandit.DelegatingHandler` will switch the Handler for the connection to be\n`Bandit.WebSocket.Handler`. This will cause any future communication after the\nupgrade process to be handled directly by Bandit's WebSocket stack.","ref":"WebSocket_README.md.html#upgrade-mechanism"},{"type":"extras","title":"Process model - WebSocket Implementation Notes","doc":"Within a Bandit server, a WebSocket connection is modeled as a single process.\nThis process is directly tied to the lifecycle of the underlying WebSocket\nconnection; when upgrading from HTTP/1, the existing HTTP/1 handler process\n'magically' becomes a WebSocket process by changing which Handler the\n`Bandit.DelegatingHandler` delegates to. \n\nThe execution model to handle a given request is quite straightforward: at\nupgrade time, the `Bandit.DelegatingHandler` will call `handle_connection/2` to\nallow the WebSocket handler to initialize any startup state. Connection state is\nmodeled by the `Bandit.WebSocket.Connection` struct and module.\n\nAll data subsequently received by the underlying [Thousand\nIsland](https://github.com/mtrudel/thousand_island) library will result in\na call to `Bandit.WebSocket.Handler.handle_data/3`, which will then attempt to\nparse the data into one or more WebSocket frames. Once a frame has been\nconstructed, it is them passed through to the configured `WebSock` handler by\nway of the underlying `Bandit.WebSocket.Connection`.\n\n# Testing\n\nAll of this is exhaustively tested. Tests are broken up primarily into `protocol_test.exs`, which\nis concerned with aspects of the implementation relating to protocol conformance and\nclient-facing concerns, while `sock_test.exs` is concerned with aspects of the implementation\nhaving to do with the WebSock API and application-facing concerns. There are also more\nunit-style tests covering frame serialization and deserialization.\n\nIn addition, the `autobahn` conformance suite is run via a `System` wrapper & executes the entirety\nof the suite against a running Bandit server.","ref":"WebSocket_README.md.html#process-model"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,49,46,50]}}